// sticky rates mp 0.75(y) 1.75(pi) 
// Dynare model file to calculate the GK model
// Created by Peter Karadi
// July 2010

parameters betta sig hh varphi zetta theta alfa G_over_Y eta_i epsilon gam gam_P kappa_pi kappa_y rho_i rho_ksi sigma_ksi rho_a sigma_a rho_g sigma_g sigma_Ne sigma_i rho_shock_psi sigma_psi kappa tau omega lambda chi b delta_c G_ss I_ss kh eps_b Gamma_w mu_w eps_w;
var mei Y Ym K Keff L I C G Q varrho Lambda Rk R N Ne Nn nu eta phi z x Pm w VMPK U X D F Z i prem delta  Welf a ksi g infl inflstar rw; %a1_w a2_w w_star;
varexo e_a e_ksi e_g e_Ne e_i e_mei;

kh=0;
betta=0.99000000;
sig=1.00000000;
hh=0.815;
varphi=0.27600000;
zetta=7.20000000;
theta=0.97155955;
alfa=0.33000000;
G_over_Y=0.20000000;
eta_i=1.72800000;
epsilon=5;
gam=0.78;
gam_P=0.24100000;
kappa_pi=3.1;
kappa_y=0;

rho_i=0.8;
rho_ksi=0.66000000;
rho_a=0.95000000;
sigma_a=0.01000000;
rho_g=0.95000000;
eps_b    = 50;
rho_shock_psi=0.66000000;
kappa=10.00000000;
tau=0.00100000;
omega=0.00222778;
lambda=0.38149499;
chi=3.41080850;
b=0.03760101;
delta_c=0.02041451;
G_ss       =   0.16975710;
I_ss       =   0.14153927;
Gamma_w    =   0.3;
eps_w      =   9.0909;
mu_w       = 0;
model;
//Home household
//1. Marginal utility of consumption
exp(varrho)  =   (exp(C)-hh*exp(C(-1)))^(-sig)-betta*hh*(exp(C(+1))-hh*exp(C))^(-sig);

//2. Euler equation
betta*exp(R)*exp(Lambda(+1))  =   1;

//3. Stochastic discount rate
exp(Lambda)  =   exp(varrho)/exp(varrho(-1));

//4. Labor market equilibrium
chi*exp(L)^varphi    =   exp(varrho)*exp(Pm)*(1-alfa)*exp(Y)/exp(L);

//Financial Intermediaries
//5. Value of banks' capital
exp(nu)     =   (1-theta)*betta*exp(Lambda(+1))*(exp(rw(+1))-exp(R))+betta*exp(Lambda(+1))*theta*exp(x(+1))*exp(nu(+1));

//6. Value of banks' net wealth
exp(eta)    =   (1-theta)+betta*exp(Lambda(+1))*theta*exp(z(+1))*exp(eta(+1));

//7. Optimal leverage
exp(phi)    =   exp(eta)/(lambda-exp(nu));

//8. Growth rate of banks' capital
exp(z)      =   (exp(rw)-exp(R(-1)))*exp(phi(-1))+exp(R(-1));

//9. Growth rate of banks' net wealth
exp(x)      =   (exp(phi)/exp(phi(-1)))*exp(z);

//Aggregate capital, net worth
//10. Aggregate capital
exp(Q)*exp(K)     =   exp(phi)*exp(N);

//11. Banks' net worth
exp(N)      =   theta*exp(z)*exp(N(-1))*exp(-e_Ne)+exp(Nn);

//12. Existing banks' net worth accumulation
exp(Ne)     =   theta*exp(z)*exp(N(-1))*exp(-e_Ne);

//13. New banks' net worth
exp(Nn)    =   omega*exp(Q)*exp(ksi)*exp(K(-1));

//Final goods producer
//14. Return to capital
exp(Rk)     =   (exp(Pm)*alfa*exp(Ym)/exp(K(-1))+exp(ksi)*(exp(Q)-exp(delta)))/exp(Q(-1));

//15. Production function
exp(Ym)     =   exp(a)*(exp(ksi)*exp(U)*exp(K(-1)))^alfa*exp(L)^(1-alfa);

//Capital Goods Producer
//16. Optimal investment decision
%exp(mei)*exp(Q) = exp(mei)+ exp(mei)*eta_i/2*(exp(I)/exp(I(-1))-1)^2+exp(mei)*eta_i*(exp(I)/exp(I(-1))-1)*exp(I)/exp(I(-1))-exp(mei(+1))*betta*eta_i*exp(Lambda(+1))*(exp(I(+1))/exp(I)-1)*(exp(I(+1))/exp(I))^2;
exp(mei)*exp(Q) = exp(mei)+ eta_i/2*(exp(I)/exp(I(-1))-1)^2+eta_i*(exp(I)/exp(I(-1))-1)*exp(I)/exp(I(-1))-betta*eta_i*exp(Lambda(+1))*(exp(I(+1))/exp(I)-1)*(exp(I(+1))/exp(I))^2;

//17. Depreciation rate
exp(delta) = delta_c+b/(1+zetta)*exp(U)^(1+zetta);

//18. Optimal capacity utilization rate
%exp(Pm)*alfa*exp(Ym)/exp(U) = b*exp(U)^zetta*exp(ksi)*exp(K(-1));
exp(U)   = 1;
//19. Net investment
%In  =   exp(I)-exp(delta)*exp(ksi)*exp(K(-1));

//20. Capital accumulation equation
%exp(K)  =   exp(ksi)*exp(K(-1))+In; 
exp(K) = exp(mei)*exp(I) + exp(ksi)*(1-exp(delta))*exp(K(-1));

//21. Government consumption
exp(G)   =   G_ss*exp(g);

//Equilibrium
//22. Aggregate resource constraint
exp(Y)   =   exp(C)+exp(G)+exp(I)+eta_i/2*((I)/(I(-1))-1)^2*(I);

//23. Wholesale, retail output
exp(Ym)    =   exp(Y)*exp(D);

//24. Price dispersion
exp(D)    =   gam*exp(D(-1))*exp(infl(-1))^(-gam_P*epsilon)*exp(infl)^epsilon+(1-gam)*((1-gam*exp(infl(-1))^(gam_P*(1-gam))*exp(infl)^(gam-1))/(1-gam))^(-epsilon/(1-gam));

//25. Markup
exp(X)  =   1/exp(Pm);

//26. Optimal price choice
exp(F)       =   exp(Y)*exp(Pm)+betta*gam*exp(Lambda(+1))*exp(infl(+1))^epsilon*(exp(infl))^(-epsilon*gam_P)*exp(F(+1));

//27.
exp(Z)       =   exp(Y)+betta*gam*exp(Lambda(+1))*exp(infl(+1))^(epsilon-1)*exp(infl)^(gam_P*(1-epsilon))*exp(Z(+1));

//28. Optimal price choice
exp(inflstar)   =  epsilon/(epsilon-1)*exp(F)/exp(Z)*exp(infl);

//29. Price index
(exp(infl))^(1-epsilon)     =   gam*exp(infl(-1))^(gam_P*(1-epsilon))+(1-gam)*(exp(inflstar))^(1-epsilon);

//30. Fisher equation
exp(i)  =   exp(R)*exp(infl(+1));

//31. Interest rate rule
exp(i)      =   exp(i(-1))^rho_i*((1/betta)*exp(infl)^kappa_pi*(exp(X)/(epsilon/(epsilon-1)))^(kappa_y))^(1-rho_i)*exp(-e_i);

//Shocks
//32. TFP shock
a  =   rho_a*a(-1)-e_a;

//33. Capital quality shock
ksi=   rho_ksi*ksi(-1)-e_ksi;

//34. Government consumption shock
g  =   rho_g*g(-1)-e_g;

//Some extra variables for convenience
//35. Effective capital
exp(Keff)   =   exp(ksi)*exp(K(-1));

//36. Wages
exp(w)      =   exp(Pm)*(1-alfa)*exp(Y)/exp(L);

//37. Marginal value product of capital
exp(VMPK)   =   exp(Pm)*alfa*exp(Y)/(exp(ksi)*exp(K(-1)));

//38. Welfare
Welf   =   log(exp(C)-hh*exp(C(-1)))-chi*exp(L)^(1+varphi)/(1+varphi)+betta*Welf(+1);

//39. Premium
exp(prem)   =   exp(rw(+1))/exp(R);

0=1-eps_b+eps_b*exp(rw)/exp(Rk)-kh*(exp(Rk)/exp(Rk(-1))-1)+(exp(Rk(+1))/exp(Rk)-1)^2*(exp(Rk)/exp(Rk(-1))-1)*exp(Lambda(+1))*betta*kh*exp(K(+1))/exp(K);

%exp(a1_w)  = exp(varrho)*exp(w)^(eps_w)*exp(L)+betta*Gamma_w*((exp(infl))^mu_w/(exp(infl(+1))))^(eps_w-1)*exp(a1_w(+1));
%51
%exp(a2_w)  = chi*exp(w)^(eps_w*(1+varphi))*exp(L)^(1+varphi)+betta*Gamma_w*((exp(infl))^mu_w/(exp(infl(+1))))^(-eps_w*(1+varphi))*exp(a2_w(+1));
%52
%exp(w)^(1-eps_w)=(1-Gamma_w)*exp(w_star)^(1-eps_w)+Gamma_w*((exp(infl(-1)))^mu_w/(exp(infl))*exp(w(-1)))^(1-eps_w);
%53
%exp(w_star)^(1+eps_w*varphi) = exp(a2_w)/exp(a1_w)*eps_w/(eps_w-1);

mei = mei(-1)*0.8+e_mei;
end;


initval;
Y=log(0.84878550);
Ym=log(0.84878550);
K=log(5.66157077);
Keff=log(5.66157077);
L=log(0.33333333);
I=log(0.14153927);
C=log(0.53748913);
G=log(1);
Q=log(1.00000000);
varrho=log(1.94246544);
Lambda=log(1.00000000);
Rk=log(1.03333);
rw=log(1.01260101);
R=log(1.01010101);
N=log(1.41539269);
Ne=log(1.40277996);
Nn=log(0.01261274);
nu=log(0.00373978);
eta=log(1.51102084);
phi=log(4.00000000);
z=log(1.02010101);
x=log(1.02010101);
Pm=log(0.76001920);
w=log(1.29663748);
VMPK=log(0.03760101);
U=log(1.00000000);
X=log(1.31575624);
D=log(1.00000000);
F=log(2.81958684);
Z=log(3.70988896);
i=log(1.01010101);
prem=log(1.00247500);
delta=log(0.02500000);
%In=0.00000000;
Welf=-296.62068821;
a=0.00000000;
ksi=0.00000000;
g=0.00000000;
infl=0.00000000;
inflstar=0.00000000;
%w_star =log(1.3404);
%a1_w = log(22.1489);
%a2_w = log(27.9343);
mei    = 0;
end;
%check;
steady;

shocks;

var e_Ne; stderr 0.05;
var e_i; stderr 0.05;
var e_ksi; stderr 0.05;
%var e_mei; stderr 0.05;
end;
stoch_simul (order=1, irf=100, nograph);